/*
 * IRC - Internet Relay Chat, ircd/m_privset.c
 * Copyright (C) 1990 Jarkko Oikarinen and
 *                    University of Oulu, Computing Center
 *
 * See file AUTHORS in IRC package for additional names of
 * the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: m_privset.c,v 1.4 2003/03/05 08:02:58 sirvulcan Exp $
 */

#include "channel.h"
#include "config.h"
#include "client.h"
#include "handlers.h"
#include "hash.h"
#include "ircd.h"
#include "ircd_defs.h"
#include "ircd_features.h"
#include "ircd_reply.h"
#include "ircd_string.h"
#include "ircd_snprintf.h"
#include "ircd_alloc.h"
#include "list.h"
#include "msg.h"
#include "numeric.h"
#include "numnicks.h"
#include "querycmds.h"
#include "querycmds.h"
#include "send.h"
#include "s_conf.h"
#include "s_user.h"
#include "s_debug.h"
#include "userload.h"
#include "patchlevel.h"

#include <string.h>
#include <ctype.h>
#include <stdlib.h>

/* Code Originally By Math- <math@rootservices.net> */

int ms_privset(struct Client *cptr, struct Client *sptr, int parc, char *parv[])
{
  struct Client* acptr;

  if(parc<3)
    return(need_more_params(sptr, "PRIVSET"));

  if (!(acptr = findNUser(parv[1])))
    return 0; /* Target doesnt exist, ignore */

  parse_privs(acptr, parv[2]);
  sendto_opmask_butone(0, SNO_OLDSNO, "%s Modified Privilages On %s [%s]", cli_name(sptr), cli_name(acptr), parv[2]);
  sendcmdto_serv_butone(sptr, CMD_PRIVSET, cptr, "%C :%s", acptr, parv[2]);

  return 0;
}


#define PRIV_DEL 0
#define PRIV_ADD 1
void parse_privs(struct Client* acptr, char *privstring) {
 char *p;
 int what = PRIV_ADD;
 for (p = privstring; *p; p++) {
  switch (*p) {
   case '+':
    what = PRIV_ADD;
    break;
   case '-':
    what = PRIV_DEL;
    break;

   case 'l':
    if (what == PRIV_ADD)
      GrantPriv(acptr, PRIV_SAPART);
    else
      RevokePriv(acptr, PRIV_SAPART);
    break;

   case 'L':
    if (what == PRIV_ADD)
      GrantPriv(acptr, PRIV_CHAN_LIMIT);
    else
      RevokePriv(acptr, PRIV_CHAN_LIMIT);
    break;

   case 'c':
    if (what == PRIV_ADD)
      GrantPriv(acptr, PRIV_SAJOIN);
    else
      RevokePriv(acptr, PRIV_SAJOIN);
    break;

   case 'C':
    if (what == PRIV_ADD)
      GrantPriv(acptr, PRIV_MODE_LCHAN);
    else
      RevokePriv(acptr, PRIV_MODE_LCHAN);
    break;

   case 'w':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_WALK_LCHAN);
     else
       RevokePriv(acptr, PRIV_WALK_LCHAN);
     break;

   case 'W':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_WALK_CHAN);
     else
       RevokePriv(acptr, PRIV_WALK_CHAN);
     break;

   case 'd':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_DEOP_LCHAN);
     else
       RevokePriv(acptr, PRIV_DEOP_LCHAN);
     break;

   case 'V':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_SHOW_INVIS);
     else
       RevokePriv(acptr, PRIV_SHOW_INVIS);
     break;


   case 'I':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_SHOW_ALL_INVIS);
     else
       RevokePriv(acptr, PRIV_SHOW_ALL_INVIS);
     break;

   case 'U':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_UNLIMIT_QUERY);
     else
       RevokePriv(acptr, PRIV_UNLIMIT_QUERY);
     break;


   case 'K':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_KILL);
     else
       RevokePriv(acptr, PRIV_KILL);
     break;

   case 'k':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_LOCAL_KILL);
     else
       RevokePriv(acptr, PRIV_LOCAL_KILL);
     break;

   case 'r':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_REHASH);
     else
       RevokePriv(acptr, PRIV_REHASH);
     break;

   case 'R':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_RESTART);
     else
       RevokePriv(acptr, PRIV_RESTART);
     break;

   case 'i':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_DIE);
     else
       RevokePriv(acptr, PRIV_DIE);
     break;

   case 'G':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_GLINE);
     else
       RevokePriv(acptr, PRIV_GLINE);
     break;

   case 'g':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_LOCAL_GLINE);
     else
       RevokePriv(acptr, PRIV_LOCAL_GLINE);
     break;

   case 'J':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_JUPE);
     else
       RevokePriv(acptr, PRIV_JUPE);
     break;

   case 'j':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_LOCAL_JUPE);
     else
       RevokePriv(acptr, PRIV_LOCAL_JUPE);
     break;

   case 'O':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_OPMODE);
     else
       RevokePriv(acptr, PRIV_OPMODE);
     break;

   case 'o':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_LOCAL_OPMODE);
     else
       RevokePriv(acptr, PRIV_LOCAL_OPMODE);
     break;

   case 'S':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_SET);
     else
       RevokePriv(acptr, PRIV_SET);
     break;

   case 'X':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_WHOX);
     else
       RevokePriv(acptr, PRIV_WHOX);
     break;

   case 'B':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_BADCHAN);
     else
       RevokePriv(acptr, PRIV_BADCHAN);
     break;

   case 'b':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_LOCAL_BADCHAN);
     else
       RevokePriv(acptr, PRIV_LOCAL_BADCHAN);
     break;

   case 'p':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_SEE_CHAN);
     else
       RevokePriv(acptr, PRIV_SEE_CHAN);
     break;

   case 'P':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_PROPAGATE);
     else
       RevokePriv(acptr, PRIV_PROPAGATE);
     break;

   case 'D':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_DISPLAY);
     else
       RevokePriv(acptr, PRIV_DISPLAY);
     break;

   case 'Y':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_SEE_OPERS);
     else
       RevokePriv(acptr, PRIV_SEE_OPERS);
     break;


   case 'N':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_WIDE_GLINE);
     else
       RevokePriv(acptr, PRIV_WIDE_GLINE);
     break;

   case 'F':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_FORCE_OPMODE);
     else
       RevokePriv(acptr, PRIV_FORCE_OPMODE);
     break;

   case 'f':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_FORCE_LOCAL_OPMODE);
     else
       RevokePriv(acptr, PRIV_FORCE_LOCAL_OPMODE);
     break;

   case 'n':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_SANICK);
     else
       RevokePriv(acptr, PRIV_SANICK);
     break;


   case 'E':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_SAIDENT);
     else
       RevokePriv(acptr, PRIV_SAIDENT);
     break;

   case 'H':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_SAHOST);
     else
       RevokePriv(acptr, PRIV_SAHOST);
     break;

   case 'A':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_SANAME);
     else
       RevokePriv(acptr, PRIV_SANAME);
     break;

   case 'a':
     if (what == PRIV_ADD)
       GrantPriv(acptr, PRIV_RAHASH);
     else
       RevokePriv(acptr, PRIV_RAHASH);
     break;

   default:
   break;
  }
 }

}
